"""add maimaipy_identifier

迁移 ID: 5c791c303fe1
父迁移: de4344fd6e4c
创建时间: 2026-01-02 22:43:32.110967

"""

from __future__ import annotations

from collections.abc import Sequence

import sqlalchemy as sa
from alembic import op

revision: str = "5c791c303fe1"
down_revision: str | Sequence[str] | None = "de4344fd6e4c"
branch_labels: str | Sequence[str] | None = None
depends_on: str | Sequence[str] | None = None


def upgrade(name: str = "") -> None:
    if name:
        return
    # ### commands auto generated by Alembic - please adjust! ###
    with op.batch_alter_table("nonebot_plugin_rikka_userbindinfo", schema=None) as batch_op:
        batch_op.add_column(sa.Column("maimaipy_identifier", sa.String(), nullable=True))

    # ### end Alembic commands ###


def downgrade(name: str = "") -> None:
    if name:
        return
    # ### commands auto generated by Alembic - please adjust! ###
    with op.batch_alter_table("nonebot_plugin_rikka_userbindinfo", schema=None) as batch_op:
        batch_op.drop_column("maimaipy_identifier")

    # ### end Alembic commands ###
